# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("hid.dll")
prototypes = \
    {
        # 
        'HidP_GetCaps': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"Usage": SimTypeShort(signed=False, label="UInt16"), "UsagePage": SimTypeShort(signed=False, label="UInt16"), "InputReportByteLength": SimTypeShort(signed=False, label="UInt16"), "OutputReportByteLength": SimTypeShort(signed=False, label="UInt16"), "FeatureReportByteLength": SimTypeShort(signed=False, label="UInt16"), "Reserved": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 17), "NumberLinkCollectionNodes": SimTypeShort(signed=False, label="UInt16"), "NumberInputButtonCaps": SimTypeShort(signed=False, label="UInt16"), "NumberInputValueCaps": SimTypeShort(signed=False, label="UInt16"), "NumberInputDataIndices": SimTypeShort(signed=False, label="UInt16"), "NumberOutputButtonCaps": SimTypeShort(signed=False, label="UInt16"), "NumberOutputValueCaps": SimTypeShort(signed=False, label="UInt16"), "NumberOutputDataIndices": SimTypeShort(signed=False, label="UInt16"), "NumberFeatureButtonCaps": SimTypeShort(signed=False, label="UInt16"), "NumberFeatureValueCaps": SimTypeShort(signed=False, label="UInt16"), "NumberFeatureDataIndices": SimTypeShort(signed=False, label="UInt16")}, name="HIDP_CAPS", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["PreparsedData", "Capabilities"]),
        # 
        'HidP_GetLinkCollectionNodes': SimTypeFunction([SimTypePointer(SimStruct({"LinkUsage": SimTypeShort(signed=False, label="UInt16"), "LinkUsagePage": SimTypeShort(signed=False, label="UInt16"), "Parent": SimTypeShort(signed=False, label="UInt16"), "NumberOfChildren": SimTypeShort(signed=False, label="UInt16"), "NextSibling": SimTypeShort(signed=False, label="UInt16"), "FirstChild": SimTypeShort(signed=False, label="UInt16"), "_bitfield": SimTypeInt(signed=False, label="UInt32"), "UserContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="HIDP_LINK_COLLECTION_NODE", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["LinkCollectionNodes", "LinkCollectionNodesLength", "PreparsedData"]),
        # 
        'HidP_GetSpecificButtonCaps': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimStruct({"UsagePage": SimTypeShort(signed=False, label="UInt16"), "ReportID": SimTypeChar(label="Byte"), "IsAlias": SimTypeBottom(label="BOOLEAN"), "BitField": SimTypeShort(signed=False, label="UInt16"), "LinkCollection": SimTypeShort(signed=False, label="UInt16"), "LinkUsage": SimTypeShort(signed=False, label="UInt16"), "LinkUsagePage": SimTypeShort(signed=False, label="UInt16"), "IsRange": SimTypeBottom(label="BOOLEAN"), "IsStringRange": SimTypeBottom(label="BOOLEAN"), "IsDesignatorRange": SimTypeBottom(label="BOOLEAN"), "IsAbsolute": SimTypeBottom(label="BOOLEAN"), "Reserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 10), "Anonymous": SimUnion({"Range": SimStruct({"UsageMin": SimTypeShort(signed=False, label="UInt16"), "UsageMax": SimTypeShort(signed=False, label="UInt16"), "StringMin": SimTypeShort(signed=False, label="UInt16"), "StringMax": SimTypeShort(signed=False, label="UInt16"), "DesignatorMin": SimTypeShort(signed=False, label="UInt16"), "DesignatorMax": SimTypeShort(signed=False, label="UInt16"), "DataIndexMin": SimTypeShort(signed=False, label="UInt16"), "DataIndexMax": SimTypeShort(signed=False, label="UInt16")}, name="_Range_e__Struct", pack=False, align=None), "NotRange": SimStruct({"Usage": SimTypeShort(signed=False, label="UInt16"), "Reserved1": SimTypeShort(signed=False, label="UInt16"), "StringIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved2": SimTypeShort(signed=False, label="UInt16"), "DesignatorIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved3": SimTypeShort(signed=False, label="UInt16"), "DataIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved4": SimTypeShort(signed=False, label="UInt16")}, name="_NotRange_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="HIDP_BUTTON_CAPS", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "Usage", "ButtonCaps", "ButtonCapsLength", "PreparsedData"]),
        # 
        'HidP_GetButtonCaps': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypePointer(SimStruct({"UsagePage": SimTypeShort(signed=False, label="UInt16"), "ReportID": SimTypeChar(label="Byte"), "IsAlias": SimTypeBottom(label="BOOLEAN"), "BitField": SimTypeShort(signed=False, label="UInt16"), "LinkCollection": SimTypeShort(signed=False, label="UInt16"), "LinkUsage": SimTypeShort(signed=False, label="UInt16"), "LinkUsagePage": SimTypeShort(signed=False, label="UInt16"), "IsRange": SimTypeBottom(label="BOOLEAN"), "IsStringRange": SimTypeBottom(label="BOOLEAN"), "IsDesignatorRange": SimTypeBottom(label="BOOLEAN"), "IsAbsolute": SimTypeBottom(label="BOOLEAN"), "Reserved": SimTypeFixedSizeArray(SimTypeInt(signed=False, label="UInt32"), 10), "Anonymous": SimUnion({"Range": SimStruct({"UsageMin": SimTypeShort(signed=False, label="UInt16"), "UsageMax": SimTypeShort(signed=False, label="UInt16"), "StringMin": SimTypeShort(signed=False, label="UInt16"), "StringMax": SimTypeShort(signed=False, label="UInt16"), "DesignatorMin": SimTypeShort(signed=False, label="UInt16"), "DesignatorMax": SimTypeShort(signed=False, label="UInt16"), "DataIndexMin": SimTypeShort(signed=False, label="UInt16"), "DataIndexMax": SimTypeShort(signed=False, label="UInt16")}, name="_Range_e__Struct", pack=False, align=None), "NotRange": SimStruct({"Usage": SimTypeShort(signed=False, label="UInt16"), "Reserved1": SimTypeShort(signed=False, label="UInt16"), "StringIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved2": SimTypeShort(signed=False, label="UInt16"), "DesignatorIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved3": SimTypeShort(signed=False, label="UInt16"), "DataIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved4": SimTypeShort(signed=False, label="UInt16")}, name="_NotRange_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="HIDP_BUTTON_CAPS", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "ButtonCaps", "ButtonCapsLength", "PreparsedData"]),
        # 
        'HidP_GetSpecificValueCaps': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimStruct({"UsagePage": SimTypeShort(signed=False, label="UInt16"), "ReportID": SimTypeChar(label="Byte"), "IsAlias": SimTypeBottom(label="BOOLEAN"), "BitField": SimTypeShort(signed=False, label="UInt16"), "LinkCollection": SimTypeShort(signed=False, label="UInt16"), "LinkUsage": SimTypeShort(signed=False, label="UInt16"), "LinkUsagePage": SimTypeShort(signed=False, label="UInt16"), "IsRange": SimTypeBottom(label="BOOLEAN"), "IsStringRange": SimTypeBottom(label="BOOLEAN"), "IsDesignatorRange": SimTypeBottom(label="BOOLEAN"), "IsAbsolute": SimTypeBottom(label="BOOLEAN"), "HasNull": SimTypeBottom(label="BOOLEAN"), "Reserved": SimTypeChar(label="Byte"), "BitSize": SimTypeShort(signed=False, label="UInt16"), "ReportCount": SimTypeShort(signed=False, label="UInt16"), "Reserved2": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 5), "UnitsExp": SimTypeInt(signed=False, label="UInt32"), "Units": SimTypeInt(signed=False, label="UInt32"), "LogicalMin": SimTypeInt(signed=True, label="Int32"), "LogicalMax": SimTypeInt(signed=True, label="Int32"), "PhysicalMin": SimTypeInt(signed=True, label="Int32"), "PhysicalMax": SimTypeInt(signed=True, label="Int32"), "Anonymous": SimUnion({"Range": SimStruct({"UsageMin": SimTypeShort(signed=False, label="UInt16"), "UsageMax": SimTypeShort(signed=False, label="UInt16"), "StringMin": SimTypeShort(signed=False, label="UInt16"), "StringMax": SimTypeShort(signed=False, label="UInt16"), "DesignatorMin": SimTypeShort(signed=False, label="UInt16"), "DesignatorMax": SimTypeShort(signed=False, label="UInt16"), "DataIndexMin": SimTypeShort(signed=False, label="UInt16"), "DataIndexMax": SimTypeShort(signed=False, label="UInt16")}, name="_Range_e__Struct", pack=False, align=None), "NotRange": SimStruct({"Usage": SimTypeShort(signed=False, label="UInt16"), "Reserved1": SimTypeShort(signed=False, label="UInt16"), "StringIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved2": SimTypeShort(signed=False, label="UInt16"), "DesignatorIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved3": SimTypeShort(signed=False, label="UInt16"), "DataIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved4": SimTypeShort(signed=False, label="UInt16")}, name="_NotRange_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="HIDP_VALUE_CAPS", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "Usage", "ValueCaps", "ValueCapsLength", "PreparsedData"]),
        # 
        'HidP_GetValueCaps': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypePointer(SimStruct({"UsagePage": SimTypeShort(signed=False, label="UInt16"), "ReportID": SimTypeChar(label="Byte"), "IsAlias": SimTypeBottom(label="BOOLEAN"), "BitField": SimTypeShort(signed=False, label="UInt16"), "LinkCollection": SimTypeShort(signed=False, label="UInt16"), "LinkUsage": SimTypeShort(signed=False, label="UInt16"), "LinkUsagePage": SimTypeShort(signed=False, label="UInt16"), "IsRange": SimTypeBottom(label="BOOLEAN"), "IsStringRange": SimTypeBottom(label="BOOLEAN"), "IsDesignatorRange": SimTypeBottom(label="BOOLEAN"), "IsAbsolute": SimTypeBottom(label="BOOLEAN"), "HasNull": SimTypeBottom(label="BOOLEAN"), "Reserved": SimTypeChar(label="Byte"), "BitSize": SimTypeShort(signed=False, label="UInt16"), "ReportCount": SimTypeShort(signed=False, label="UInt16"), "Reserved2": SimTypeFixedSizeArray(SimTypeShort(signed=False, label="UInt16"), 5), "UnitsExp": SimTypeInt(signed=False, label="UInt32"), "Units": SimTypeInt(signed=False, label="UInt32"), "LogicalMin": SimTypeInt(signed=True, label="Int32"), "LogicalMax": SimTypeInt(signed=True, label="Int32"), "PhysicalMin": SimTypeInt(signed=True, label="Int32"), "PhysicalMax": SimTypeInt(signed=True, label="Int32"), "Anonymous": SimUnion({"Range": SimStruct({"UsageMin": SimTypeShort(signed=False, label="UInt16"), "UsageMax": SimTypeShort(signed=False, label="UInt16"), "StringMin": SimTypeShort(signed=False, label="UInt16"), "StringMax": SimTypeShort(signed=False, label="UInt16"), "DesignatorMin": SimTypeShort(signed=False, label="UInt16"), "DesignatorMax": SimTypeShort(signed=False, label="UInt16"), "DataIndexMin": SimTypeShort(signed=False, label="UInt16"), "DataIndexMax": SimTypeShort(signed=False, label="UInt16")}, name="_Range_e__Struct", pack=False, align=None), "NotRange": SimStruct({"Usage": SimTypeShort(signed=False, label="UInt16"), "Reserved1": SimTypeShort(signed=False, label="UInt16"), "StringIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved2": SimTypeShort(signed=False, label="UInt16"), "DesignatorIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved3": SimTypeShort(signed=False, label="UInt16"), "DataIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved4": SimTypeShort(signed=False, label="UInt16")}, name="_NotRange_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="HIDP_VALUE_CAPS", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "ValueCaps", "ValueCapsLength", "PreparsedData"]),
        # 
        'HidP_GetExtendedAttributes': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"NumGlobalUnknowns": SimTypeChar(label="Byte"), "Reserved": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "GlobalUnknowns": SimTypePointer(SimStruct({"Token": SimTypeChar(label="Byte"), "Reserved": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 3), "BitField": SimTypeInt(signed=False, label="UInt32")}, name="HIDP_UNKNOWN_TOKEN", pack=False, align=None), offset=0), "Data": SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)}, name="HIDP_EXTENDED_ATTRIBUTES", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "DataIndex", "PreparsedData", "Attributes", "LengthAttributes"]),
        # 
        'HidP_InitializeReportForID': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "ReportID", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_SetData': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypePointer(SimStruct({"DataIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"RawValue": SimTypeInt(signed=False, label="UInt32"), "On": SimTypeBottom(label="BOOLEAN")}, name="<anon>", label="None")}, name="HIDP_DATA", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "DataList", "DataLength", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_GetData': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypePointer(SimStruct({"DataIndex": SimTypeShort(signed=False, label="UInt16"), "Reserved": SimTypeShort(signed=False, label="UInt16"), "Anonymous": SimUnion({"RawValue": SimTypeInt(signed=False, label="UInt32"), "On": SimTypeBottom(label="BOOLEAN")}, name="<anon>", label="None")}, name="HIDP_DATA", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "DataList", "DataLength", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_MaxDataListLength': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ReportType", "PreparsedData"]),
        # 
        'HidP_SetUsages': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "UsageList", "UsageLength", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_UnsetUsages': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "UsageList", "UsageLength", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_GetUsages': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "UsageList", "UsageLength", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_GetUsagesEx': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimStruct({"Usage": SimTypeShort(signed=False, label="UInt16"), "UsagePage": SimTypeShort(signed=False, label="UInt16")}, name="USAGE_AND_PAGE", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "LinkCollection", "ButtonList", "UsageLength", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_MaxUsageListLength': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ReportType", "UsagePage", "PreparsedData"]),
        # 
        'HidP_SetUsageValue': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "Usage", "UsageValue", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_SetScaledUsageValue': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "Usage", "UsageValue", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_SetUsageValueArray': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "Usage", "UsageValue", "UsageValueByteLength", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_GetUsageValue': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "Usage", "UsageValue", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_GetScaledUsageValue': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int32"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "Usage", "UsageValue", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_GetUsageValueArray': SimTypeFunction([SimTypeInt(signed=False, label="HIDP_REPORT_TYPE"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeShort(signed=False, label="UInt16"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["ReportType", "UsagePage", "LinkCollection", "Usage", "UsageValue", "UsageValueByteLength", "PreparsedData", "Report", "ReportLength"]),
        # 
        'HidP_UsageListDifference': SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0), SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["PreviousUsageList", "CurrentUsageList", "BreakUsageList", "MakeUsageList", "UsageListLength"]),
        # 
        'HidP_TranslateUsagesToI8042ScanCodes': SimTypeFunction([SimTypePointer(SimTypeShort(signed=False, label="UInt16"), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="HIDP_KEYBOARD_DIRECTION"), SimTypePointer(SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"_bitfield": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "ul": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None")}, name="HIDP_KEYBOARD_MODIFIER_STATE", pack=False, align=None), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeBottom(label="PSTR"), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="BOOLEAN"), arg_names=["Context", "NewScanCodes", "Length"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ChangedUsageList", "UsageListLength", "KeyAction", "ModifierState", "InsertCodesProcedure", "InsertCodesContext"]),
        # 
        'HidD_GetAttributes': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "VendorID": SimTypeShort(signed=False, label="UInt16"), "ProductID": SimTypeShort(signed=False, label="UInt16"), "VersionNumber": SimTypeShort(signed=False, label="UInt16")}, name="HIDD_ATTRIBUTES", pack=False, align=None), offset=0)], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "Attributes"]),
        # 
        'HidD_GetHidGuid': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeBottom(label="Void"), arg_names=["HidGuid"]),
        # 
        'HidD_GetPreparsedData': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "PreparsedData"]),
        # 
        'HidD_FreePreparsedData': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeChar(label="Byte"), arg_names=["PreparsedData"]),
        # 
        'HidD_FlushQueue': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject"]),
        # 
        'HidD_GetConfiguration': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cookie": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "size": SimTypeInt(signed=False, label="UInt32"), "RingBufferSize": SimTypeInt(signed=False, label="UInt32")}, name="HIDD_CONFIGURATION", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "Configuration", "ConfigurationLength"]),
        # 
        'HidD_SetConfiguration': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"cookie": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "size": SimTypeInt(signed=False, label="UInt32"), "RingBufferSize": SimTypeInt(signed=False, label="UInt32")}, name="HIDD_CONFIGURATION", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "Configuration", "ConfigurationLength"]),
        # 
        'HidD_GetFeature': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "ReportBuffer", "ReportBufferLength"]),
        # 
        'HidD_SetFeature': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "ReportBuffer", "ReportBufferLength"]),
        # 
        'HidD_GetInputReport': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "ReportBuffer", "ReportBufferLength"]),
        # 
        'HidD_SetOutputReport': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "ReportBuffer", "ReportBufferLength"]),
        # 
        'HidD_GetNumInputBuffers': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "NumberBuffers"]),
        # 
        'HidD_SetNumInputBuffers': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "NumberBuffers"]),
        # 
        'HidD_GetPhysicalDescriptor': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "Buffer", "BufferLength"]),
        # 
        'HidD_GetManufacturerString': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "Buffer", "BufferLength"]),
        # 
        'HidD_GetProductString': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "Buffer", "BufferLength"]),
        # 
        'HidD_GetIndexedString': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "StringIndex", "Buffer", "BufferLength"]),
        # 
        'HidD_GetSerialNumberString': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "Buffer", "BufferLength"]),
        # 
        'HidD_GetMsGenreDescriptor': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeChar(label="Byte"), arg_names=["HidDeviceObject", "Buffer", "BufferLength"]),
    }

lib.set_prototypes(prototypes)



###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_PIC_12_LE_16_PIC_12C5xx(ArchPcode):
    name = 'PIC-12:LE:16:PIC-12C5xx'
    pcode_arch = 'PIC-12:LE:16:PIC-12C5xx'
    description = 'PIC-12C5xx'
    bits = 16
    ip_offset = 0x0
    sp_offset = 0x2
    bp_offset = sp_offset
    instruction_endness = Endness.LE
    register_list = [
        Register('indf', 1, 0x0),
        Register('tmr0', 1, 0x1),
        Register('pcl.0', 1, 0x2),
        Register('status.0', 1, 0x3),
        Register('fsr.0', 1, 0x4),
        Register('osccal', 1, 0x5),
        Register('gpio', 1, 0x6),
        Register('pc', 2, 0x0, alias_names=('ip',)),
        Register('stkptr', 1, 0x2),
        Register('w', 1, 0x3),
        Register('pcl', 1, 0x4),
        Register('fsr', 1, 0x5),
        Register('status', 1, 0x6),
        Register('pa', 1, 0x7),
        Register('z', 1, 0x8),
        Register('dc', 1, 0x9),
        Register('c', 1, 0xa),
        Register('option', 1, 0xb),
        Register('tris', 1, 0x20)
    ]

register_arch(['pic-12:le:16:pic-12c5xx'], 16, Endness.LE, ArchPcode_PIC_12_LE_16_PIC_12C5xx)



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("gpedit.dll")
prototypes = \
    {
        # 
        'CreateGPOLink': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["lpGPO", "lpContainer", "fHighPriority"]),
        # 
        'DeleteGPOLink': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpGPO", "lpContainer"]),
        # 
        'DeleteAllGPOLinks': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpContainer"]),
        # 
        'BrowseForGPO': SimTypeFunction([SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwFlags": SimTypeInt(signed=False, label="UInt32"), "hwndOwner": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "lpTitle": SimTypePointer(SimTypeChar(label="Char"), offset=0), "lpInitialOU": SimTypePointer(SimTypeChar(label="Char"), offset=0), "lpDSPath": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwDSPathSize": SimTypeInt(signed=False, label="UInt32"), "lpName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "dwNameSize": SimTypeInt(signed=False, label="UInt32"), "gpoType": SimTypeInt(signed=False, label="GROUP_POLICY_OBJECT_TYPE"), "gpoHint": SimTypeInt(signed=False, label="GROUP_POLICY_HINT_TYPE")}, name="GPOBROWSEINFO", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpBrowseInfo"]),
        # 
        'ImportRSoPData': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpNameSpace", "lpFileName"]),
        # 
        'ExportRSoPData': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["lpNameSpace", "lpFileName"]),
    }

lib.set_prototypes(prototypes)



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("rstrtmgr.dll")
prototypes = \
    {
        # 
        'RmStartSession': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pSessionHandle", "dwSessionFlags", "strSessionKey"]),
        # 
        'RmJoinSession': SimTypeFunction([SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pSessionHandle", "strSessionKey"]),
        # 
        'RmEndSession': SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSessionHandle"]),
        # 
        'RmRegisterResources': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"dwProcessId": SimTypeInt(signed=False, label="UInt32"), "ProcessStartTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None)}, name="RM_UNIQUE_PROCESS", pack=False, align=None), label="LPArray", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), label="LPArray", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSessionHandle", "nFiles", "rgsFileNames", "nApplications", "rgApplications", "nServices", "rgsServiceNames"]),
        # 
        'RmGetList': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimStruct({"Process": SimStruct({"dwProcessId": SimTypeInt(signed=False, label="UInt32"), "ProcessStartTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None)}, name="RM_UNIQUE_PROCESS", pack=False, align=None), "strAppName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 256), "strServiceShortName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 64), "ApplicationType": SimTypeInt(signed=False, label="RM_APP_TYPE"), "AppStatus": SimTypeInt(signed=False, label="UInt32"), "TSSessionId": SimTypeInt(signed=False, label="UInt32"), "bRestartable": SimTypeInt(signed=True, label="Int32")}, name="RM_PROCESS_INFO", pack=False, align=None), label="LPArray", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSessionHandle", "pnProcInfoNeeded", "pnProcInfo", "rgAffectedApps", "lpdwRebootReasons"]),
        # 
        'RmShutdown': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["nPercentComplete"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSessionHandle", "lActionFlags", "fnStatus"]),
        # 
        'RmRestart': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["nPercentComplete"]), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSessionHandle", "dwRestartFlags", "fnStatus"]),
        # 
        'RmCancelCurrentTask': SimTypeFunction([SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSessionHandle"]),
        # 
        'RmAddFilter': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwProcessId": SimTypeInt(signed=False, label="UInt32"), "ProcessStartTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None)}, name="RM_UNIQUE_PROCESS", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="RM_FILTER_ACTION")], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSessionHandle", "strModuleName", "pProcess", "strServiceShortName", "FilterAction"]),
        # 
        'RmRemoveFilter': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"dwProcessId": SimTypeInt(signed=False, label="UInt32"), "ProcessStartTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None)}, name="RM_UNIQUE_PROCESS", pack=False, align=None), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSessionHandle", "strModuleName", "pProcess", "strServiceShortName"]),
        # 
        'RmGetFilterList': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["dwSessionHandle", "pbFilterBuf", "cbFilterBuf", "cbFilterBufNeeded"]),
    }

lib.set_prototypes(prototypes)



from .heavy import HeavyVEXMixin
from .inspect import SimInspectMixin
from .actions import TrackActionsMixin
from .resilience import HeavyResilienceMixin
from .super_fastpath import SuperFastpathMixin


###
### This file was automatically generated
###

from archinfo.arch import register_arch, Endness, Register

from .common import ArchPcode


class ArchPcode_80251_BE_24_default(ArchPcode):
    name = '80251:BE:24:default'
    pcode_arch = '80251:BE:24:default'
    description = '80251 Microcontroller Family'
    bits = 16
    ip_offset = 0x44
    sp_offset = 0x3c
    bp_offset = sp_offset
    instruction_endness = Endness.BE
    register_list = [
        Register('dr0', 4, 0x0),
        Register('wr0', 2, 0x0),
        Register('r0', 1, 0x0),
        Register('r1', 1, 0x1),
        Register('wr2', 2, 0x2),
        Register('r2', 1, 0x2),
        Register('r3', 1, 0x3),
        Register('dr4', 4, 0x4),
        Register('wr4', 2, 0x4),
        Register('r4', 1, 0x4),
        Register('r5', 1, 0x5),
        Register('wr6', 2, 0x6),
        Register('r6', 1, 0x6),
        Register('r7', 1, 0x7),
        Register('dr8', 4, 0x8),
        Register('wr8', 2, 0x8),
        Register('r8', 1, 0x8),
        Register('r9', 1, 0x9),
        Register('ab', 2, 0xa),
        Register('b', 1, 0xa),
        Register('acc', 1, 0xb),
        Register('dr12', 4, 0xc),
        Register('wr12', 2, 0xc),
        Register('r12', 1, 0xc),
        Register('r13', 1, 0xd),
        Register('wr14', 2, 0xe),
        Register('r14', 1, 0xe),
        Register('r15', 1, 0xf),
        Register('dr16', 4, 0x10),
        Register('wr16', 2, 0x10),
        Register('r16', 1, 0x10),
        Register('r17', 1, 0x11),
        Register('wr18', 2, 0x12),
        Register('r18', 1, 0x12),
        Register('r19', 1, 0x13),
        Register('dr20', 4, 0x14),
        Register('wr20', 2, 0x14),
        Register('r20', 1, 0x14),
        Register('r21', 1, 0x15),
        Register('wr22', 2, 0x16),
        Register('r22', 1, 0x16),
        Register('r23', 1, 0x17),
        Register('dr24', 4, 0x18),
        Register('wr24', 2, 0x18),
        Register('r24', 1, 0x18),
        Register('r25', 1, 0x19),
        Register('wr26', 2, 0x1a),
        Register('r26', 1, 0x1a),
        Register('r27', 1, 0x1b),
        Register('dr28', 4, 0x1c),
        Register('wr28', 2, 0x1c),
        Register('r28', 1, 0x1c),
        Register('r29', 1, 0x1d),
        Register('wr30', 2, 0x1e),
        Register('r30', 1, 0x1e),
        Register('r31', 1, 0x1f),
        Register('dpx', 4, 0x38),
        Register('r56', 1, 0x38),
        Register('dpxl', 1, 0x39),
        Register('dptr', 2, 0x3a),
        Register('dph', 1, 0x3a),
        Register('dpl', 1, 0x3b),
        Register('spx', 4, 0x3c),
        Register('r60', 1, 0x3c),
        Register('r61', 1, 0x3d),
        Register('sph', 1, 0x3e),
        Register('sp', 3, 0x40),
        Register('pc', 3, 0x44, alias_names=('ip',)),
        Register('psw', 1, 0x48),
        Register('contextreg', 4, 0x50),
        Register('jumptableguard1', 1, 0x70),
        Register('jumptableguard2', 1, 0x71)
    ]

register_arch(['80251:be:24:default'], 16, Endness.BE, ArchPcode_80251_BE_24_default)



from typing import Optional
import logging

import claripy

from ..knowledge_plugins.cfg import CFGModel
from ..analyses.cfg import CFGUtils
from . import Analysis, register_analysis

_l = logging.getLogger(name=__name__)


class CompleteCallingConventionsAnalysis(Analysis):

    def __init__(self, recover_variables=False, low_priority=False, force=False, cfg: Optional[CFGModel]=None,
                 analyze_callsites: bool=False, skip_signature_matched_functions: bool=False):

        self._recover_variables = recover_variables
        self._low_priority = low_priority
        self._force = force
        self._cfg = cfg
        self._analyze_callsites = analyze_callsites
        self._skip_signature_matched_functions = skip_signature_matched_functions

        self._analyze()

    def _analyze(self):
        """
        Infer calling conventions for all functions in the current project.

        :return:
        """

        # get an ordering of functions based on the call graph
        sorted_funcs = CFGUtils.quasi_topological_sort_nodes(self.kb.functions.callgraph)
        total_funcs = len(sorted_funcs)

        self._update_progress(0)

        for idx, func_addr in enumerate(reversed(sorted_funcs)):
            func = self.kb.functions.get_by_addr(func_addr)

            if func.calling_convention is None or self._force:
                if func.alignment:
                    # skip all alignments
                    continue

                if self._skip_signature_matched_functions and func.from_signature:
                    # this function matches against a known library function. skip it.
                    continue

                # if it's a normal function, we attempt to perform variable recovery
                if self._recover_variables and self.function_needs_variable_recovery(func):
                    _l.info("Performing variable recovery on %r...", func)
                    try:
                        _ = self.project.analyses.VariableRecoveryFast(func, kb=self.kb, low_priority=self._low_priority)
                    except claripy.ClaripyError:
                        _l.warning("An claripy exception occurred during variable recovery analysis on function %#x.",
                                   func.addr,
                                   exc_info=True,
                                   )
                        continue

                # determine the calling convention of each function
                cc_analysis = self.project.analyses.CallingConvention(func, cfg=self._cfg,
                                                                      analyze_callsites=self._analyze_callsites)
                if cc_analysis.cc is not None:
                    _l.info("Determined calling convention for %r.", func)
                    func.calling_convention = cc_analysis.cc
                else:
                    _l.info("Cannot determine calling convention for %r.", func)

            percentage = (idx + 1) / total_funcs * 100.0
            self._update_progress(percentage)
            if self._low_priority:
                self._release_gil(idx, 1, 0.000001)

    #
    # Static methods
    #

    @staticmethod
    def function_needs_variable_recovery(func):
        """
        Check if running variable recovery on the function is the only way to determine the calling convention of the
        this function.

        We do not need to run variable recovery to determine the calling convention of a function if:
        - The function is a SimProcedure.
        - The function is a PLT stub.
        - The function is a library function and we already know its prototype.

        :param func:    The function object.
        :return:        True if we must run VariableRecovery before we can determine what the calling convention of this
                        function is. False otherwise.
        :rtype:         bool
        """

        if func.is_simprocedure or func.is_plt:
            return False
        # TODO: Check SimLibraries
        return True


register_analysis(CompleteCallingConventionsAnalysis, "CompleteCallingConventions")
